#include "lcd.h"


#define DELAY_PERSCALER     4
GPIO_InitTypeDef GPIO_InitStructure;

/* 8 user defined characters to be loaded into CGRAM (used for bargraph)      */
const char UserFont[8][8] = {
  { 0x11,0x0A,0x04,0x1B,0x11,0x11,0x11,0x0E },
  { 0x10,0x10,0x10,0x10,0x10,0x10,0x10,0x10 },
  { 0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18 },
  { 0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C,0x1C },
  { 0x1E,0x1E,0x1E,0x1E,0x1E,0x1E,0x1E,0x1E },
  { 0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F },
  { 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 },
  { 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00 }
};

/*************************************************************************************************************/
static void lcd_delay_us(unsigned int cnt){
  cnt <<= DELAY_PERSCALER;
  while (cnt--);
}

/*************************************************************************************************************/
static void lcd_writeNible(unsigned char val){   
  if(val & 0x01)	
    LCD_D4(1);  
  else
    LCD_D4(0);
  
  if(val & 0x02)  
    LCD_D5(1);  
  else
    LCD_D5(0);
 
  if(val & 0x04)  
    LCD_D6(1);  
  else
    LCD_D6(0);
  
  if(val & 0x08) 
    LCD_D7(1);   
  else
    LCD_D7(0); 
}

/*************************************************************************************************************/
static char busy_lcd(void){  
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;		// Config D7 = Read
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_InitStructure.GPIO_Pin = LCD_D7_PIN;
  GPIO_Init(LCD_DATA_PORT, &GPIO_InitStructure);

  LCD_RS(0);	// Instruction Select
  LCD_RW(1); 	// Read Direction
  LCD_EN(1);	// Start Read Busy
  lcd_delay_us(2);
  
  if (GPIO_ReadInputDataBit(LCD_DATA_PORT, LCD_D7_PIN) == Bit_SET){
    lcd_delay_us(2);
    LCD_EN(0);  		// Disable Read
  	LCD_RW(0);			// Default = Write Direction
  	
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;  	// Config D7 = Write
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Pin = LCD_D7_PIN;
    GPIO_Init(LCD_DATA_PORT, &GPIO_InitStructure);
    return 1;											// LCD Busy Status
  }
  else{
    lcd_delay_us(2);
    LCD_EN(0);  		// Disable Read
  	LCD_RW(0);			// Default = Write Direction
  	
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;	// Config D7 = Write
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Pin = LCD_D7_PIN;
    GPIO_Init(LCD_DATA_PORT, &GPIO_InitStructure);
    return 0;											// LCD Ready Status
  }  
}

/*************************************************************************************************************/
void lcd_cmd(unsigned char cmd){ 
  LCD_RS(0);	// RS=0 --> Instruction Select
 
  LCD_EN(1); 
  lcd_writeNible((cmd>>4) & 0x0F);
  lcd_delay_us(2);
  LCD_EN(0);

  lcd_delay_us(2);
   
  LCD_EN(1); 
  lcd_writeNible(cmd & 0x0F);
  lcd_delay_us(2);
  LCD_EN(0); 

  lcd_delay_us(2);  
  while(busy_lcd());      								// Wait LCD Execute Complete 
}

/*************************************************************************************************************/
void lcd_putchar(unsigned char ch){  
  LCD_RS(1);	// RS=1 --> Data Select
 
  LCD_EN(1); 
  lcd_writeNible((ch>>4) & 0x0F);
  lcd_delay_us(2);
  LCD_EN(0);

  lcd_delay_us(2);
   
  LCD_EN(1); 
  lcd_writeNible(ch & 0x0F);
  lcd_delay_us(2);
  LCD_EN(0); 

  lcd_delay_us(2);  
  while(busy_lcd());      								// Wait LCD Execute Complete 
}

/*************************************************************************************************************/
void lcd_init(void){
  int i;
  char const *p;
  lcd_delay_us(30000);    // Power-On delay
  
  /* Configure IO connected to LCD16X2 */
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIO_EN, ENABLE);
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Pin = LCD_EN_PIN;
  GPIO_Init(LCD_EN_PORT, &GPIO_InitStructure);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIO_RW, ENABLE);
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Pin = LCD_RW_PIN;
  GPIO_Init(LCD_RW_PORT, &GPIO_InitStructure);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIO_RS, ENABLE);
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Pin = LCD_RS_PIN;
  GPIO_Init(LCD_RS_PORT, &GPIO_InitStructure);

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIO_DATA, ENABLE);
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Pin = LCD_D4_PIN |
                                LCD_D5_PIN |
								LCD_D6_PIN |
                                LCD_D7_PIN;
  GPIO_Init(LCD_DATA_PORT, &GPIO_InitStructure);
  
  LCD_EN(0);    // lcd_disable
  LCD_RW(0);    // write mode
  LCD_RS(0);    // command mode

  lcd_cmd(0x03);
  lcd_cmd(0x03);
  lcd_cmd(0x03);
  lcd_delay_us(100);
   
  lcd_cmd(0x02);      
  lcd_cmd(0x28);  		// Function Set (DL=0 4-Bit,N=1 2 Line,F=0 5X7)
  lcd_cmd(0x0C);  		// Display on/off Control (Entry Display,Cursor off,Cursor not Blink)
  lcd_cmd(0x06);  		// Entry Mode Set (I/D=1 Increment,S=0 Cursor Shift)
  lcd_cmd(0x01);  		// Clear Display  (Clear Display,Set DD RAM Address=0) 
  lcd_delay_us(100);  	// Wait Command Ready  

  /* Load user-specific characters into CGRAM                                 */
  lcd_cmd(0x40);                  /* Set CGRAM address counter to 0     */
  p = &UserFont[0][0];
  for (i = 0; i < sizeof(UserFont); i++, p++)
    lcd_putchar(*p);

  lcd_cmd(0x80);                  /* Set DDRAM address counter to 0     */
}

/*************************************************************************************************************/
void lcd_gotoxy(char x, char y){   //Cursor to X Y position
  char DDRAMAddr;
  switch(y){
     case 1: DDRAMAddr = 0x00+x; break;
     case 2: DDRAMAddr = 0x40+x; break;
     case 3: DDRAMAddr = 0x14+x; break;
     case 4: DDRAMAddr = 0x54+x; break;
     default: DDRAMAddr = 0x00+x;
  }
  lcd_cmd(1<<7 | DDRAMAddr);
  lcd_delay_us(10);
}

/*************************************************************************************************************/
void lcd_puts(char *str){
  while(*str){
	lcd_putchar(*str);
    str++;	
  }
}
 
/*************************************************************************************************************/
void lcd_putsf(const char *str){   //Outputs string to LCD
  while(*str){
	lcd_putchar(*str);
    str++;	
  }
}

/*************************************************************************************************************/
void lcd_putInt(int data){
  char str[8] = {0,0,0,0,0,0,0,0};
  int16_t i=0, j=0;

  //Handle negative integers
  if(data < 0){
    lcd_putchar('-');   //Write Negative sign
    data *= -1;          //convert to positive
  }

  do{
    str[i] = (data % 10) + '0';
    data /= 10;
		i++;
  }while(data);
	
	j = i - 1;
  do{
	  lcd_putchar(str[j]);
		j--;
	}while(j>=0);
}

/*************************************************************************************************************/
void lcd_clear(void){
	lcd_cmd(0x01);     	// Clear Display
  lcd_delay_us(10000);
}

/*************************************************************************************************************/

