#ifndef __LCD_H
#define __LCD_H

#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"

/************************************ << LCD CONNECTIONS >> *********************************/
#define  LCD_RS_PIN		               GPIO_Pin_12	
#define  LCD_RS_PORT		           GPIOB
#define  RCC_APB2Periph_GPIO_RS        RCC_APB2Periph_GPIOB

#define  LCD_RW_PIN		               GPIO_Pin_13		
#define  LCD_RW_PORT	               GPIOB
#define  RCC_APB2Periph_GPIO_RW        RCC_APB2Periph_GPIOB

#define  LCD_EN_PIN 		       	   GPIO_Pin_14	
#define  LCD_EN_PORT		           GPIOB
#define  RCC_APB2Periph_GPIO_EN        RCC_APB2Periph_GPIOB

#define  LCD_D4_PIN 		       	   GPIO_Pin_8		
#define  LCD_D5_PIN 		       	   GPIO_Pin_9		
#define  LCD_D6_PIN 		       	   GPIO_Pin_10		
#define  LCD_D7_PIN 		       	   GPIO_Pin_11	
#define  LCD_DATA_PORT	               GPIOA
#define  LCD_DATA_POS 	               3	//data port starts at PA3
#define  RCC_APB2Periph_GPIO_DATA      RCC_APB2Periph_GPIOA

#define  LCD_EN(x)    	               (x ? GPIO_SetBits(LCD_EN_PORT, LCD_EN_PIN) : GPIO_ResetBits(LCD_EN_PORT, LCD_EN_PIN))  
#define  LCD_RW(x)    	               (x ? GPIO_SetBits(LCD_RW_PORT, LCD_RW_PIN) : GPIO_ResetBits(LCD_RW_PORT, LCD_RW_PIN))    
#define  LCD_RS(x) 	                   (x ? GPIO_SetBits(LCD_RS_PORT, LCD_RS_PIN) : GPIO_ResetBits(LCD_RS_PORT, LCD_RS_PIN))  

#define  LCD_D4(x) 	                   (x ? GPIO_SetBits(LCD_DATA_PORT, LCD_D4_PIN) : GPIO_ResetBits(LCD_DATA_PORT, LCD_D4_PIN)) 
#define  LCD_D5(x) 	                   (x ? GPIO_SetBits(LCD_DATA_PORT, LCD_D5_PIN) : GPIO_ResetBits(LCD_DATA_PORT, LCD_D5_PIN))
#define  LCD_D6(x) 	                   (x ? GPIO_SetBits(LCD_DATA_PORT, LCD_D6_PIN) : GPIO_ResetBits(LCD_DATA_PORT, LCD_D6_PIN)) 
#define  LCD_D7(x) 	                   (x ? GPIO_SetBits(LCD_DATA_PORT, LCD_D7_PIN) : GPIO_ResetBits(LCD_DATA_PORT, LCD_D7_PIN)) 

/* Exported macro ------------------------------------------------------------------------------------------------------ */
#define  lcd_cursor_home()             lcd_cmd(0x02)	// Set Cursor = 0
#define  lcd_display_on()              lcd_cmd(0x0E)	// LCD Display Enable
#define  lcd_display_off()             lcd_cmd(0x08)	// LCD Display Disable
#define  lcd_cursor_blink()            lcd_cmd(0x0F)	// Set Cursor = Blink
#define  lcd_cursor_on()               lcd_cmd(0x0E)	// Enable LCD Cursor
#define  lcd_cursor_off()              lcd_cmd(0x0C)	// Disable LCD Cursor
#define  lcd_cursor_left()             lcd_cmd(0x10)	// Shift Left Cursor
#define  lcd_cursor_right()            lcd_cmd(0x14)	// Shift Right Cursor
#define  lcd_display_sleft()           lcd_cmd(0x18)	// Shift Left Display
#define  lcd_display_sright()          lcd_cmd(0x1C)	// Shift Right Display
  
/* Exported functions ------------------------------------------------------------------------------------------------ */
void lcd_cmd(unsigned char cmd);
void lcd_putchar(unsigned char ch);
void lcd_init(void);
void lcd_gotoxy(char x, char y);
void lcd_puts(char *str);
void lcd_putsf(const char *str);
void lcd_putInt(int data);
void lcd_clear(void);



#endif /* __LCD_H */
